/************************************************************************
 * "Modified by Turbon, Version 0.01.00, 4-9-12 for WM8741 DAC Project" *
 ************************************************************************/
/*
Hifiduino v 0.7a
May 19, 2011: Minor update and bug fixes to the version 0.7 of June 21, 2009

---------------------------------------------------------------------------
----------------------------------------------------------------------------

In this release:
- Support for Web4robot LCD: http://www.arduino.cc/playground/Code/LCDAPI
- Large number display for filter selection and volume
- Digital Filter Selection: Filter 1 to Filter 5.
- Rotary encoder.
- Volume Control. Now -99 db to 0db in 1db increment.
- Volume "dimmer". Volume dims to -60 dB when pushing rotary encoder
- Changed the default delay in the LCD library to (0,0) See below note.
- Select input sample rate (low, med, high) for future use.
- Adjust brightness and contrast of display with remote. Not used
- Remember brightness and contrast settings in eeprom. Not used
- New fonts for large number display
*/

// LIBRARIES

#include <LiquidCrystal.h> // For LCD 
// Initialize the library with the numbers of the interface pins 
LiquidCrystal lcd(12, 11, 10, 9, 8, 7); 
#include <Wire.h> // For I2C (What pins?)
// We will use the eeprom to store values
#include <EEPROM.h>

// Define register values, etc to facilitate programming
// Note: Reg address is address plus R/W bit
// This is why the address here is different from the
// address in the data sheet

#define REG9 0x12     // Register 9 address (Reset)
#define RESET 0x00    // Write anything to reset the DAC

#define REG7 0x0E     // Register 7 address (PCM vs DSD)
#define PCMHIGH 0x40  // Value for reg7 for PCM, High Sample rate for filter
#define PCMMED 0x20  // Value for reg7 for PCM, Medium Sample rate for filter
#define PCMLOW 0x00  // Value for reg7 for PCM, Low Sample rate for filter

#define REG6 0x0C     // Register 6 address (filters)
#define FILTER1 0x00  // PCM Filter response 1
#define FILTER2 0x01  // PCM Filter response 2
#define FILTER3 0x02  // PCM Filter response 3
#define FILTER4 0x03  // PCM Filter response 4
#define FILTER5 0x04  // PCM Filter response 5

#define REG0 0x00     // Register 0 address (Volume Adjustment)
#define REG1 0x02     // Register 1 address (Volume Adjustment)
#define REG2 0x04     // Register 2 address (Volume Adjustment)
#define REG3 0x06     // Register 3 address (Volume Adjustment)
#define REG4 0x08     // Register 4 address (Volume Control)

// The following to be used in Reg4
//#define VOLCNTR 0x07 // VolRight=VolLeft, AntiClip, Vol Ramping is on (a)
#define VOLCNTR 0x05  // VolRight=VolLeft, No AntiClip, Vol Ramping is on (b)
//#define MUTE 0x0F // Mute and (a) above
#define MUTE 0x0D     // Mute and (b) above

//#define DEFAULTVOL 0x190 //-50 dB this is 50x8=400
#define DEFAULTVOL 0x00 //-50 dB this is 50x8=400
#define MINVOL 0x318    //-99dB this is 99X8=792. -Dac adjustment is .125 db
#define MAXVOL 0x00     //-0 dB - Min attenuation
#define DIMVOL 0x1E0    //-60dB The volume level when dimming the volume

#define FILTERPIN 6     // Button to select Filter
#define VOLDIMPIN 5     // Button for volume dimming feature
#define VOLUPPIN 4      // Button to increase volume
#define VOLDOWNPIN 2    // Button to decrease volume

#define REMOTEPIN 3         // The pin for the remote sensor


//APPLE Code

// The following variables for the remote control feature
int duration;         // Duration of the IR pulse
int mask;             // Used to decode the remote signals 
int c1;               // Byte 1 of the 32-bit remote command code
int c2;               // Byte 2 of the 32-bit remote command code
int c3;               // Byte 3 of the 32-bit remote command code
int c4;               // Byte 4 of the 32-bit remote command code
int IRkey;            // The unique code (Byte 3) of the remote key
int previousIRkey;    // The previous code (used for repeat)

#define IRFILSELDELAY 300  // Delay when selecting filter with remote to preven multiple selection
#define ONEPULSE 1000      // Microsecond threshold for value 1 in remote putlse

#define B 0xFF             // The character for a completely filled box
#define A 0x20             // The character for blank

// The routine to create the custom characters in the LCD

  // A 1 in the binary representation of the character means it is filled in
  // characters are 5 pixels wide by 8 pixels tall

  // We need 7 custom characters for the OPUS DAC display.
  // (Custom character 0 doesn't work in the Web4Robot LCD)

  // Define Custom Characters

byte cc0[8] = {     // Custom Character 0
  B00000,
  B00111,
  B01111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111
};

byte cc1[8] = {     // Custom Character 1
  B11100,
  B11110,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111
};

byte cc2[8] = {    // Custom Character 2
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B00000,
  B00000,
  B00000
};

byte cc3[8] = {    // Custom Character 3
  B00000,
  B00000,
  B00000,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111
};

byte cc4[8] = {   // Custom Character 4
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B01111,
  B00111
};

byte cc5[8] = {    // Custom Character 5
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B00000,
  B00000
};

byte cc6[8] = {    // Custom Character 6
  B00000,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B00000,
  B00000
};

byte cc7[8] = {     // Custom Character 7
  B00000,
  B11100,
  B11110,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111
};

// Function to send custom characters to the display's RAM
void DefineCustomChar(){
  lcd.createChar(0,cc0);  // cc0 becomes character 0
  lcd.createChar(1,cc1);  // cc1 becomes character 1
  lcd.createChar(2,cc2);  // cc2 becomes character 2
  lcd.createChar(3,cc3);  // cc3 becomes character 3
  lcd.createChar(4,cc4);  // cc4 becomes character 4
  lcd.createChar(5,cc5);  // cc5 becomes character 5
  lcd.createChar(6,cc6);  // cc6 becomes character 6
  lcd.createChar(7,cc7);  // cc7 becomes character 7
};

// Array index into parts of big numbers. Each number consists of 9 custom characters
// in a 3x3 matrix. To print a number, you use the array index corresponding to the number
// times 3. For example to print the number 5, you will print bn1[15], bn1[16] and bn1[17]
// for the first row of the large number, and then bn2[15], bn2[16] and bn2[17] and so on.

//            0      1      2      3      4      5      6      7      8      9    
char bn1[]={B,2,1, 2,1,A, 2,2,1, 2,2,1, 0,A,B, B,2,2, B,2,2, 2,2,B, B,2,1, B,2,1};
char bn2[]={B,A,B, A,B,A ,0,6,5, A,2,1, 5,6,B, 2,2,1, B,6,7, A,0,5, B,6,B, 5,6,B};
char bn3[]={4,3,B, 3,B,3, B,3,3, 4,3,B, A,A,B, 4,3,B, 4,3,B, A,B,A, 4,3,B, A,A,B};

void printOneNumber(uint8_t digit)
{
  // Print position is hardcoded

  // Line 1 of the one digit number
  lcd.setCursor(0,1);
  lcd.write(bn1[digit*3]);
  lcd.write(bn1[digit*3+1]);
  lcd.write(bn1[digit*3+2]);

  // Line 2 of the one-digit number
  lcd.setCursor(0,2);
  lcd.write(bn2[digit*3]);
  lcd.write(bn2[digit*3+1]);
  lcd.write(bn2[digit*3+2]);

  // Line 3 of the one-digit number
  lcd.setCursor(0,3);
  lcd.write(bn3[digit*3]);
  lcd.write(bn3[digit*3+1]);
  lcd.write(bn3[digit*3+2]);
}

// Functions for printing two large digits. Value is the column number
// and number is the number to print. Works from 00-99

void printTwoNumber(byte value, byte number){

  lcd.setCursor(value,1);  // Printing line 1 of the two-digit number
  lcd.write(bn1[(number/10)*3]);
  lcd.write(bn1[(number/10)*3+1]);
  lcd.write(bn1[(number/10)*3+2]);
  lcd.write(A); // Blank
  lcd.write(bn1[(number%10)*3]);
  lcd.write(bn1[(number%10)*3+1]);
  lcd.write(bn1[(number%10)*3+2]);

  lcd.setCursor(value,2);  // Printing line 2 of the two-digit number
  lcd.write(bn2[(number/10)*3]);
  lcd.write(bn2[(number/10)*3+1]);
  lcd.write(bn2[(number/10)*3+2]);
  lcd.write(A); // Blank
  lcd.write(bn2[(number%10)*3]);
  lcd.write(bn2[(number%10)*3+1]);
  lcd.write(bn2[(number%10)*3+2]);

  lcd.setCursor(value,3);  // Printing line 3 of the two-digit number
  lcd.write(bn3[(number/10)*3]);
  lcd.write(bn3[(number/10)*3+1]);
  lcd.write(bn3[(number/10)*3+2]);
  lcd.write(A); // Blank
  lcd.write(bn3[(number%10)*3]);
  lcd.write(bn3[(number%10)*3+1]);
  lcd.write(bn3[(number%10)*3+2]);

};

// The write to WM8741 DAC routine
void opuswritereg(uint8_t regaddr,uint8_t regval)
{
  Wire.beginTransmission(0x1B);
  Wire.write(regaddr);
  Wire.write(regval);
  Wire.endTransmission();
}

// Interrupt service routine for rotary encoder. Determines direction.

volatile byte volUp=0;  // flags for the interrupt routine
volatile byte volDown=0;

void decoder()
{
  if (digitalRead(2) == digitalRead(4))
  {
    volUp = 1;  //if on interrupt the encoder channels are the same, direction is clockwise
  }
  else
  {
    volDown = 1;  //if they are not the same, direction is ccw
  }
}

// interrupt service routine for remote

volatile byte remoteOn = 0;  // 1 means remote has been pressed

void remoting()
{
  remoteOn=1;
}

// Declaring some more variables

int currVol=DEFAULTVOL; // this needs to be at least 10 bits
byte filter=0;          // variable for current filter
byte reg0val=0;         // variable for volume lower bits
byte reg1val=0;         // variable for volume upper bits
byte volDimState=0;     // State for volume dimmer. =1 is dimmed
byte BC=0;              // State for selecting contrast or brightness
                        // BC=0, no adjustment, BC=1" Brighness
                        // BC=2: contrast
int brightness=0;
int contrast=0;

int irCode=0;           // The code returned by the remote IR pulses

/*
The following function returns the code from the Apple Aluminum remote control. The Apple remote is
 based on the NEC infrared remote protocol. Of the 32 bits (4 bytes) coded in the protocol, only the
 third byte corresponds to the keys. The function also handles errors due to noise (returns 255) and
 the repeat code (returs zero)
 
 The Apple remote returns the following codes:
 
   Up key:     238 135 011 089
   Down key:   238 135 013 089
   Left key:   238 135 008 089
   Right key:  238 135 007 089
   Center key: 238 135 093 089 followed by 238 135 004 089 (See blog for why there are two codes)
   Menu key:   238 135 002 089
   Play key:   238 135 094 089 followed by 238 135 004 089
 */

int getIRkey() {
  c1=0;
  c2=0;
  c3=0;
  c4=0;
  duration=1;
  while((duration=pulseIn(REMOTEPIN, HIGH, 15000)) < 2000 && duration!=0)
  {
    // Wait for start pulse
  }
  if (duration == 0)         // This is an error no start or end of pulse
    return(255);             // Use 255 as Error

  else if (duration<3000)    // This is the repeat
    return (0);              // Use zero as the repeat code

  else if (duration<5000){   // This is the command get the 4 byte
    mask = 1;				    
    for (int i = 0; i < 8; i++){	             // get 8 bits
      if(pulseIn(REMOTEPIN, HIGH, 3000)>1000)     // If "1" pulse
        c1 |= mask;			             // Put the "1" in position
      mask <<= 1;				     // shift mask to next bit
    }
    mask = 1;				    
    for (int i = 0; i < 8; i++){	             // get 8 bits
      if(pulseIn(REMOTEPIN, HIGH, 3000)>1000)     // If "1" pulse
        c2 |= mask;			             // Put the "1" in position
      mask <<= 1;				     // shift mask to next bit
    }
    mask = 1;				    
    for (int i = 0; i < 8; i++){	             // get 8 bits
      if(pulseIn(REMOTEPIN, HIGH, 3000)>1000)     // If "1" pulse
        c3 |= mask;			             // Put the "1" in position
      mask <<= 1;				     // shift mask to next bit
    }
    mask = 1;				    
    for (int i = 0; i < 8; i++){	             // get 8 bits
      if(pulseIn(REMOTEPIN, HIGH, 3000)>1000)     // If "1" pulse
        c4 |= mask;			             // Put the "1" in position
      mask <<= 1;				     // shift mask to next bit
    }
    // Serial.println(c1, HEX); //For debugging
    // Serial.println(c2, HEX); //For debugging
    // Serial.println(c3, HEX); //For debugging
    // Serial.println(c4, HEX); //For debugging
    return(c3);
  }
}


void setup() {
 // lcd.init();     // Initializing the LCD, clears the display
  lcd.begin(20, 4);    // Set up the LCD's number of columns and rows:

 // lcd.setDelay(0,0); // Change de delay in the LCD library 
  DefineCustomChar();  // Define large characters

  Wire.begin();   // Joining the I2C bus as master

  pinMode(FILTERPIN, INPUT);      // Button to select Input fiter
  digitalWrite(FILTERPIN, HIGH);  // Enable pull-up resistor

  pinMode(VOLUPPIN, INPUT);       // Button or Encoder pin for volume up
  digitalWrite(VOLUPPIN, HIGH);   // Enable pull-up resistor

  pinMode(VOLDOWNPIN, INPUT);     // Button or Encoder pin for volume down
  digitalWrite(VOLDOWNPIN, HIGH); // Enable pull-down resistor

  pinMode(VOLDIMPIN, INPUT);      // Button or switch for volume dimmer
  digitalWrite(VOLDIMPIN, HIGH);  // Enable pull-up resistor

  pinMode(REMOTEPIN, INPUT);          // Pin for IR Receiver
  digitalWrite(REMOTEPIN, HIGH);      // Enable high as specified by datasheet

  // Print the welcome message and other labels to the LCD

  lcd.clear();
  lcd.setCursor(1,1);
  lcd.print("H I F I D U I N O");
  lcd.setCursor(2,2);
  lcd.print("v.8741-0.2  STEREO");
  lcd.setCursor(4,3);
  lcd.print("April 09 2012");
 
  delay(3000);
  
  lcd.clear();

  opuswritereg(REG9,RESET);   // Resetting the DAC just in case
  opuswritereg(REG4,MUTE);    // MUTE (and vol setup) 

  // The following 4 lines is to set up the default volume
  // To set up the volume you need to write to at least 2 registers

  reg0val=(DEFAULTVOL & 0x1F);     // The lower 5 bits goes to reg 0
  opuswritereg(REG0,reg0val);      // Writing to reg 0
  reg1val=((DEFAULTVOL>>5)| 0x20); // The upper 5 bits goes to reg 1
  opuswritereg(REG1,reg1val);      // Writing to reg 1

  opuswritereg(REG6,FILTER1);  // I'm setting the default filter at power on
  opuswritereg(REG7,PCMHIGH);  // Setting up for PCM and high sampling rate because I have 
                               // a reclocker and the output is 192K to the DAC 
  opuswritereg(REG4,VOLCNTR);  // Setting up volume control. Unmute

  //attachInterrupt(0, decoder, CHANGE);  // ISR for rotary encoder
  attachInterrupt(0, decoder, RISING);  // ISR for rotary encoder
  attachInterrupt(1, remoting, RISING); // ISR for remote IR sensor

  lcd.setCursor(0,0);
  // print header for digital filter and current filter 
  lcd.print("LnS");
  //printTwoNumber(13,1);
  printOneNumber(1);

  // print header for volume and volume
  lcd.setCursor(13,0);
  lcd.print("-dB VOL");
  printTwoNumber(13, currVol/8);

  // Print the rest of the display
  lcd.setCursor(4 ,2);
  lcd.print("INPT");
  lcd.write(0xA5);
  lcd.print("PCM");
  lcd.setCursor(4,1);
  lcd.print("RATE");
  lcd.write(0xA5);
  lcd.print("192");

}

void loop()
{
  // The following code will select the next filter if the button is pushed
  // and it is repeated for the remote because I'm lazy  
  
  if (digitalRead(FILTERPIN)==0)  // When the filter button is pushed
  {
    filter++;
    filter=filter%5;
        switch(filter){
      case 0:
      lcd.setCursor(0,0);
      lcd.print("LnS");
      opuswritereg(REG6,FILTER1);
      printOneNumber(1);
      break;
      case 1:
      lcd.setCursor(0,0);
      lcd.print("MnS");
      opuswritereg(REG6,FILTER2);
      printOneNumber(2);
      break;
      case 2:
      lcd.setCursor(0,0);
      lcd.print("BrK");
      opuswritereg(REG6,FILTER3);
      printOneNumber(3);
      break;
      case 3:
      lcd.setCursor(0,0);
      lcd.print("MnA");
      opuswritereg(REG6,FILTER4);
      printOneNumber(4);
      break;
      case 4:
      lcd.setCursor(0,0);
      lcd.print("LnA");
      opuswritereg(REG6,FILTER5);
      printOneNumber(5);
      break;
    }
    delay(300);  // Used to avoid reading the button more than one during the click
  }

  // The following is to adjust the volume down (larger number)

  while(volDown==1)  // While there is CCW motion in the rotary encoder
                     // This means decrease volume -increase attenuation
  {
    delay(5);   // Debouncing delay - maybe in the wrong place...
    volDown=0;  // Reset the flag
    if (currVol<MINVOL) // If current volume is not at MIN (attn < maxAttn)
                        // then decrease volume (increase attenuation)
    {
      currVol=currVol+8;                    // Increase attenuation 1 dB
      reg0val=(((byte)currVol) & 0x1F);     // Calculate value for reg0
      opuswritereg(REG0,reg0val);           // Write to reg0
      reg1val=(((byte)(currVol>>5))| 0x20); // Calculate value for reg1
      opuswritereg(REG1,reg1val);           // Write to reg1

      printTwoNumber(13, currVol/8);

      if (volDimState==1)                // If in dim state, reset state value
      {
        volDimState=0;
        lcd.setCursor(17,0);
        lcd.print("VOL");
      }
    }
  }

  // The following is to adjust the volume up (smaller numbers)

  while(volUp==1)  // While there is cw motion in rotary encoder
  {
    delay(5); // Debouncing delay - maybe in the wrong place...
    volUp=0;  // clear the flag
    if (currVol>MAXVOL)   // If current volume not at MAX (attn>minAttn)
                          // increase volume (reduce attennuation)
    {
      currVol=currVol-8;                // Decrease attenuation 1 dB
      reg0val=(((byte)currVol) & 0x1F); // Calculate value for reg0
      opuswritereg(REG0,reg0val);       // Write to reg0
      reg1val=((currVol>>5)| 0x20);     // Calculate value for reg1
      opuswritereg(REG1,reg1val);       // Write to reg1

      printTwoNumber(13, currVol/8);

      if (volDimState==1)               // If in dim state, reset state value
      {
        volDimState=0;
        lcd.setCursor(17,0);
        lcd.print("VOL");
      }
    }
  }

  // The following is for the volume dimming feature

  if (digitalRead(VOLDIMPIN)==0) // When pressing the volume dim button
  {
    if (volDimState==0)  //if not dimmed, then dim the volume to min vol
    {
      reg0val=(((byte)DIMVOL) & 0x1F);
      opuswritereg(REG0,reg0val);
      reg1val=(((byte)(DIMVOL>>5))| 0x20);
      opuswritereg(REG1,reg1val);
      printTwoNumber(13, DIMVOL/8);
      volDimState=1;     // set state to dimmed
      lcd.setCursor(17,0);
      lcd.print("DIM");
    }
    else                 // if dimmed, then return to current volume
    {
      reg0val=(((byte)currVol) & 0x1F);
      opuswritereg(REG0,reg0val);
      reg1val=(((byte)(currVol>>5))| 0x20); // Calculate value for reg1
      opuswritereg(REG1,reg1val);
      printTwoNumber(13, currVol/8);
      volDimState=0;    // set state to undimmed.
      lcd.setCursor(17,0);
      lcd.print("VOL");
    }
    delay(300); // Used to avoid reading the button more than one during the click
  }

  while(digitalRead(REMOTEPIN)==LOW){
    if((IRkey=getIRkey())==255){
      // Do nothing
    }
    else { 
      if(IRkey==0) {            // Repeat code                           
        if(previousIRkey==11||previousIRkey==13) {
          IRkey=previousIRkey;  // Repeat code, only for specified keys. Add additional keys to 
                                // or statement in is if condition
        }
        else {
          // Do nothing. No repeat for the rest of the keys
        }
      }
      else {                    // Not a repeat code, it is a new command
        previousIRkey=IRkey;    // Remember the key in case we want to use the repeat code
      }
    }
      
    switch(IRkey){
      // The following for volume
      //case KEYVOLUP:
      case 11: // 11 is the up key, we will use for volume up
      if (currVol>MAXVOL) // If volume not at max vol, then increse vol
                          // volume is increased by decreasing the attenuation value
        {
        currVol=currVol-8;                // Decrease attenuation 1 dB
        reg0val=(((byte)currVol) & 0x1F); // Calculate value for reg0
        opuswritereg(REG0,reg0val);       // Write to reg0
        reg1val=((currVol>>5)| 0x20);     // Calculate value for reg1
        opuswritereg(REG1,reg1val);       // Write to reg1

        printTwoNumber(13, currVol/8);

        if (volDimState==1)               // If in dim state, reset state value
        {
          volDimState=0;
          lcd.setCursor(17,0);
          lcd.print("VOL");
        }
      }
      break;
      
      //case KEYVOLDOWN:
      case 13: // 13 is the down key, we will use for volume down
      if (currVol<MINVOL)  // If not at min volume, then can decrease volume
        {                  // volume is decreased by increasing the attenuation value
        currVol=currVol+8;                    // Increase attenuation 1 dB
        reg0val=(((byte)currVol) & 0x1F);     // Calculate value for reg0
        opuswritereg(REG0,reg0val);           // Write to reg0
        reg1val=(((byte)(currVol>>5))| 0x20); // Calculate value for reg1
        opuswritereg(REG1,reg1val);           // Write to reg1

        printTwoNumber(13, currVol/8);

        if (volDimState==1)                // If in dim state, reset state value
        {
          volDimState=0;
          lcd.setCursor(17,0);
          lcd.print("VOL");
        }
      }
      break;
      
      case 4:
        {
      if (volDimState==0)  //if not dimmed, then dim the volume to min vol
          {
          reg0val=(((byte)DIMVOL) & 0x1F);
          opuswritereg(REG0,reg0val);
          reg1val=(((byte)(DIMVOL>>5))| 0x20);
          opuswritereg(REG1,reg1val);
          printTwoNumber(13, DIMVOL/8);
          volDimState=1;     // set state to dimmed
          lcd.setCursor(17,0);
          lcd.print("DIM");
          }
        else                 // if dimmed, then return to current volume
          {
          reg0val=(((byte)currVol) & 0x1F);
          opuswritereg(REG0,reg0val);
          reg1val=(((byte)(currVol>>5))| 0x20); // Calculate value for reg1
          opuswritereg(REG1,reg1val);
          printTwoNumber(13, currVol/8);
          volDimState=0;    // set state to undimmed.
          lcd.setCursor(17,0);
          lcd.print("VOL");
          }
      delay(300); // Used to avoid reading the button more than one during the click
        }

  }
 }
}
